package com.sun.imageio.plugins.bmp;

import com.sun.imageio.plugins.common.ImageUtil;
import javax.imageio.metadata.IIOMetadataNode;
import java.io.UnsupportedEncodingException;
import com.sun.imageio.plugins.common.I18N;
import org.w3c.dom.Node;
import java.util.List;
import javax.imageio.metadata.IIOMetadata;

public class BMPMetadata extends IIOMetadata implements BMPConstants
{
    public static final String nativeMetadataFormatName = "javax_imageio_bmp_1.0";
    public String bmpVersion;
    public int width;
    public int height;
    public short bitsPerPixel;
    public int compression;
    public int imageSize;
    public int xPixelsPerMeter;
    public int yPixelsPerMeter;
    public int colorsUsed;
    public int colorsImportant;
    public int redMask;
    public int greenMask;
    public int blueMask;
    public int alphaMask;
    public int colorSpace;
    public double redX;
    public double redY;
    public double redZ;
    public double greenX;
    public double greenY;
    public double greenZ;
    public double blueX;
    public double blueY;
    public double blueZ;
    public int gammaRed;
    public int gammaGreen;
    public int gammaBlue;
    public int intent;
    public byte[] palette;
    public int paletteSize;
    public int red;
    public int green;
    public int blue;
    public List comments;
    
    public BMPMetadata() {
        super(true, "javax_imageio_bmp_1.0", "com.sun.imageio.plugins.bmp.BMPMetadataFormat", null, (String[])null);
        this.palette = null;
        this.comments = null;
    }
    
    @Override
    public boolean isReadOnly() {
        return true;
    }
    
    @Override
    public Node getAsTree(String s) {
        if (s.equals("javax_imageio_bmp_1.0")) {
            return this.getNativeTree();
        }
        if (s.equals("javax_imageio_1.0")) {
            return this.getStandardTree();
        }
        throw new IllegalArgumentException(I18N.getString("BMPMetadata0"));
    }
    
    private String toISO8859(byte[] bytes) {
        try {
            return new String(bytes, "ISO-8859-1");
        }
        catch (UnsupportedEncodingException ex) {
            return "";
        }
    }
    
    private Node getNativeTree() {
        IIOMetadataNode iioMetadataNode = new IIOMetadataNode("javax_imageio_bmp_1.0");
        this.addChildNode(iioMetadataNode, "BMPVersion", this.bmpVersion);
        this.addChildNode(iioMetadataNode, "Width", new Integer(this.width));
        this.addChildNode(iioMetadataNode, "Height", new Integer(this.height));
        this.addChildNode(iioMetadataNode, "BitsPerPixel", new Short(this.bitsPerPixel));
        this.addChildNode(iioMetadataNode, "Compression", new Integer(this.compression));
        this.addChildNode(iioMetadataNode, "ImageSize", new Integer(this.imageSize));
        IIOMetadataNode addChildNode = this.addChildNode(iioMetadataNode, "PixelsPerMeter", null);
        this.addChildNode(addChildNode, "X", new Integer(this.xPixelsPerMeter));
        this.addChildNode(addChildNode, "Y", new Integer(this.yPixelsPerMeter));
        this.addChildNode(iioMetadataNode, "ColorsUsed", new Integer(this.colorsUsed));
        this.addChildNode(iioMetadataNode, "ColorsImportant", new Integer(this.colorsImportant));
        int n = 0;
        for (int i = 0; i < this.bmpVersion.length(); ++i) {
            if (Character.isDigit(this.bmpVersion.charAt(i))) {
                n = this.bmpVersion.charAt(i) - 48;
            }
        }
        if (n >= 4) {
            IIOMetadataNode addChildNode2 = this.addChildNode(iioMetadataNode, "Mask", null);
            this.addChildNode(addChildNode2, "Red", new Integer(this.redMask));
            this.addChildNode(addChildNode2, "Green", new Integer(this.greenMask));
            this.addChildNode(addChildNode2, "Blue", new Integer(this.blueMask));
            this.addChildNode(addChildNode2, "Alpha", new Integer(this.alphaMask));
            this.addChildNode(iioMetadataNode, "ColorSpaceType", new Integer(this.colorSpace));
            IIOMetadataNode addChildNode3 = this.addChildNode(iioMetadataNode, "CIEXYZEndPoints", null);
            this.addXYZPoints(addChildNode3, "Red", this.redX, this.redY, this.redZ);
            this.addXYZPoints(addChildNode3, "Green", this.greenX, this.greenY, this.greenZ);
            this.addXYZPoints(addChildNode3, "Blue", this.blueX, this.blueY, this.blueZ);
            this.addChildNode(iioMetadataNode, "Intent", new Integer(this.intent));
        }
        if (this.palette != null && this.paletteSize > 0) {
            IIOMetadataNode addChildNode4 = this.addChildNode(iioMetadataNode, "Palette", null);
            int n2 = this.palette.length / this.paletteSize;
            int j = 0;
            int n3 = 0;
            while (j < this.paletteSize) {
                IIOMetadataNode addChildNode5 = this.addChildNode(addChildNode4, "PaletteEntry", null);
                this.red = (this.palette[n3++] & 0xFF);
                this.green = (this.palette[n3++] & 0xFF);
                this.blue = (this.palette[n3++] & 0xFF);
                this.addChildNode(addChildNode5, "Red", new Byte((byte)this.red));
                this.addChildNode(addChildNode5, "Green", new Byte((byte)this.green));
                this.addChildNode(addChildNode5, "Blue", new Byte((byte)this.blue));
                if (n2 == 4) {
                    this.addChildNode(addChildNode5, "Alpha", new Byte((byte)(this.palette[n3++] & 0xFF)));
                }
                ++j;
            }
            return iioMetadataNode;
        }
        return iioMetadataNode;
    }
    
    @Override
    protected IIOMetadataNode getStandardChromaNode() {
        if (this.palette != null && this.paletteSize > 0) {
            IIOMetadataNode iioMetadataNode = new IIOMetadataNode("Chroma");
            IIOMetadataNode newChild = new IIOMetadataNode("Palette");
            int i = this.palette.length / this.paletteSize;
            newChild.setAttribute("value", "" + i);
            int j = 0;
            int n = 0;
            while (j < this.paletteSize) {
                IIOMetadataNode newChild2 = new IIOMetadataNode("PaletteEntry");
                newChild2.setAttribute("index", "" + j);
                newChild2.setAttribute("red", "" + (int)this.palette[n++]);
                newChild2.setAttribute("green", "" + (int)this.palette[n++]);
                newChild2.setAttribute("blue", "" + (int)this.palette[n++]);
                if (i == 4 && this.palette[n] != 0) {
                    newChild2.setAttribute("alpha", "" + (int)this.palette[n++]);
                }
                newChild.appendChild(newChild2);
                ++j;
            }
            iioMetadataNode.appendChild(newChild);
            return iioMetadataNode;
        }
        return null;
    }
    
    @Override
    protected IIOMetadataNode getStandardCompressionNode() {
        IIOMetadataNode iioMetadataNode = new IIOMetadataNode("Compression");
        IIOMetadataNode newChild = new IIOMetadataNode("CompressionTypeName");
        newChild.setAttribute("value", BMPCompressionTypes.getName(this.compression));
        iioMetadataNode.appendChild(newChild);
        return iioMetadataNode;
    }
    
    @Override
    protected IIOMetadataNode getStandardDataNode() {
        IIOMetadataNode iioMetadataNode = new IIOMetadataNode("Data");
        String string = "";
        if (this.bitsPerPixel == 24) {
            string = "8 8 8 ";
        }
        else if (this.bitsPerPixel == 16 || this.bitsPerPixel == 32) {
            string = "" + this.countBits(this.redMask) + " " + this.countBits(this.greenMask) + this.countBits(this.blueMask) + "" + this.countBits(this.alphaMask);
        }
        IIOMetadataNode newChild = new IIOMetadataNode("BitsPerSample");
        newChild.setAttribute("value", string);
        iioMetadataNode.appendChild(newChild);
        return iioMetadataNode;
    }
    
    @Override
    protected IIOMetadataNode getStandardDimensionNode() {
        if ((float)this.yPixelsPerMeter > 0.0f && (float)this.xPixelsPerMeter > 0.0f) {
            IIOMetadataNode iioMetadataNode = new IIOMetadataNode("Dimension");
            float f = (float)(this.yPixelsPerMeter / this.xPixelsPerMeter);
            IIOMetadataNode newChild = new IIOMetadataNode("PixelAspectRatio");
            newChild.setAttribute("value", "" + f);
            iioMetadataNode.appendChild(newChild);
            IIOMetadataNode newChild2 = new IIOMetadataNode("HorizontalPhysicalPixelSpacing");
            newChild2.setAttribute("value", "" + 1 / this.xPixelsPerMeter * 1000);
            iioMetadataNode.appendChild(newChild2);
            IIOMetadataNode newChild3 = new IIOMetadataNode("VerticalPhysicalPixelSpacing");
            newChild3.setAttribute("value", "" + 1 / this.yPixelsPerMeter * 1000);
            iioMetadataNode.appendChild(newChild3);
            return iioMetadataNode;
        }
        return null;
    }
    
    @Override
    public void setFromTree(String s, Node node) {
        throw new IllegalStateException(I18N.getString("BMPMetadata1"));
    }
    
    @Override
    public void mergeTree(String s, Node node) {
        throw new IllegalStateException(I18N.getString("BMPMetadata1"));
    }
    
    @Override
    public void reset() {
        throw new IllegalStateException(I18N.getString("BMPMetadata1"));
    }
    
    private String countBits(int i) {
        int j = 0;
        while (i > 0) {
            if ((i & 0x1) == 0x1) {
                ++j;
            }
            i >>>= 1;
        }
        return (j == 0) ? "" : ("" + j);
    }
    
    private void addXYZPoints(IIOMetadataNode iioMetadataNode, String s, double value, double value2, double value3) {
        IIOMetadataNode addChildNode = this.addChildNode(iioMetadataNode, s, null);
        this.addChildNode(addChildNode, "X", new Double(value));
        this.addChildNode(addChildNode, "Y", new Double(value2));
        this.addChildNode(addChildNode, "Z", new Double(value3));
    }
    
    private IIOMetadataNode addChildNode(IIOMetadataNode iioMetadataNode, String nodeName, Object o) {
        IIOMetadataNode newChild = new IIOMetadataNode(nodeName);
        if (o != null) {
            newChild.setUserObject(o);
            newChild.setNodeValue(ImageUtil.convertObjectToString(o));
        }
        iioMetadataNode.appendChild(newChild);
        return newChild;
    }
}
